package com.elab.mq.listener;

import com.aliyun.openservices.ons.api.Action;
import com.aliyun.openservices.ons.api.ConsumeContext;
import com.aliyun.openservices.ons.api.Message;
import com.aliyun.openservices.ons.api.MessageListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 订阅组件适配器
 *
 * @author ： liukx
 * @time ： 2019/4/22 - 17:59
 */
@Component
public class MessageListenerWrapper implements MessageListener {

    @Autowired(required = false)
    private List<AbstractMessageListener> messageListeners;

    @Override
    public Action consume(Message message, ConsumeContext consumeContext) {
        if (messageListeners != null) {
            try {
                for (int i = 0; i < messageListeners.size(); i++) {
                    AbstractMessageListener abstractMessageListener = messageListeners.get(i);
                    String currentTopic = message.getTopic();
                    String topic = abstractMessageListener.topic();
                    if (topic.equals(currentTopic)) {
                        abstractMessageListener.consume(message, consumeContext);
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
                return Action.ReconsumeLater;
            }
        }
        return Action.CommitMessage;
    }
}
